@@ -1,62 +1,20 @@
 package Pcore::App::Controller::ExtJS::App;
 
-use Pcore -role, -const, -status;
+use Pcore -role, -status;
 use Pcore::Util::Text qw[mark_raw];
 use Pcore::Util::Data qw[to_json];
-use Pcore::Share::WWW;
 use Pcore::App::API qw[:CONST];
+use Pcore::Share::WWW;
+use WWW::CDN;
 
 with qw[Pcore::App::Controller::ExtJS];
 
 requires qw[ext_class_viewport];
 
-const our $EXT_FRAMEWORK_CLASSIC => 1;
-const our $EXT_FRAMEWORK_MODERN  => 2;
-
-const our $EXT_FRAMEWORK_NAME => {
-    $EXT_FRAMEWORK_CLASSIC => 'classic',
-    $EXT_FRAMEWORK_MODERN  => 'modern',
-};
-
-const our $EXT_THEME_ARIA          => 1;
-const our $EXT_THEME_CLASSIC       => 2;
-const our $EXT_THEME_CRISP         => 3;
-const our $EXT_THEME_CRISP_TOUCH   => 4;
-const our $EXT_THEME_GRAY          => 5;
-const our $EXT_THEME_NEPTUNE       => 6;
-const our $EXT_THEME_NEPTUNE_TOUCH => 7;
-const our $EXT_THEME_TRITON        => 8;
-const our $EXT_THEME_CUPERTINO     => 9;
-const our $EXT_THEME_IOS           => 10;
-const our $EXT_THEME_MATERIAL      => 11;
-const our $EXT_THEME_MOUNTAINVIEW  => 12;
-const our $EXT_THEME_WINDOWS       => 13;
-
-const our $EXT_FRAMEWORK_CLASSIC_THEME => {
-    $EXT_THEME_ARIA          => 'aria',
-    $EXT_THEME_CLASSIC       => 'classic',
-    $EXT_THEME_CRISP         => 'crisp',
-    $EXT_THEME_CRISP_TOUCH   => 'crisp-touch',
-    $EXT_THEME_GRAY          => 'gray',
-    $EXT_THEME_NEPTUNE       => 'neptune',
-    $EXT_THEME_NEPTUNE_TOUCH => 'neptune-toouch',
-    $EXT_THEME_TRITON        => 'triton',
-};
-
-const our $EXT_FRAMEWORK_MODERN_THEME => {
-    $EXT_THEME_CUPERTINO    => 'cupertino',
-    $EXT_THEME_IOS          => 'ios',
-    $EXT_THEME_MATERIAL     => 'material',
-    $EXT_THEME_MOUNTAINVIEW => 'mountainview',
-    $EXT_THEME_WINDOWS      => 'windows',
-    $EXT_THEME_NEPTUNE      => 'neptune',
-    $EXT_THEME_TRITON       => 'triton',
-};
-
-has framework => ( is => 'ro', isa => Enum [ $EXT_FRAMEWORK_CLASSIC, $EXT_FRAMEWORK_MODERN ], default => $EXT_FRAMEWORK_CLASSIC );
-has classic_theme => ( is => 'ro', isa => Enum [ keys $EXT_FRAMEWORK_CLASSIC_THEME->%* ], default => $EXT_THEME_TRITON );
-has modern_theme  => ( is => 'ro', isa => Enum [ keys $EXT_FRAMEWORK_MODERN_THEME->%* ],  default => $EXT_THEME_MATERIAL );
-has locale => ( is => 'ro', isa => Str, default => 'en' );
+has extjs_ver                   => ( is => 'ro', isa => Str, default => $WWW::CDN::LATEST_VERION->{extjs} );
+has extjs_default_theme_classic => ( is => 'ro', isa => Str, default => 'triton' );
+has extjs_default_theme_modern  => ( is => 'ro', isa => Str, default => 'triton' );
+has extjs_default_locale        => ( is => 'ro', isa => Str, default => 'en' );
 
 around run => sub ( $orig, $self, $req ) {
     if ( $req->{path_tail} && $req->{path_tail} ne 'index.html' ) {
@@ -158,12 +116,41 @@ around run => sub ( $orig, $self, $req ) {
         return;
     }
 
-    my $devel = 1;
+    my $resources = [];
 
-    my $ext_framework_name = $self->framework == $EXT_FRAMEWORK_MODERN ? 'modern-' : q[];
-    my $framework          = $EXT_FRAMEWORK_NAME->{ $self->framework };
-    my $debug              = $devel ? '-debug' : q[];
-    my $theme              = $self->framework == $EXT_FRAMEWORK_MODERN ? $EXT_FRAMEWORK_MODERN_THEME->{ $self->modern_theme } : $EXT_FRAMEWORK_CLASSIC_THEME->{ $self->classic_theme };
+    # FontAwesome
+    push $resources->@*, WWW::CDN->fontawesome;
+
+    # ExtJS framework
+    my $devel     = $req->{env}->{QUERY_STRING} =~ /\bdevel\b/sm;
+    my $ver       = $self->extjs_ver;
+    my $framework = 'classic';
+
+    my $extjs_resources;
+
+    # get theme from query
+    if ( $req->{env}->{QUERY_STRING} =~ /\btheme=([a-z-]+)/sm ) {
+        my $theme = $1;
+
+        $extjs_resources = WWW::CDN->extjs( $ver, $framework, $theme, $devel );
+    }
+
+    # fallback to the default theme
+    if ( !$extjs_resources ) {
+        my $theme = $framework eq 'classic' ? $self->extjs_default_theme_classic : $self->extjs_default_theme_modern;
+
+        $extjs_resources = WWW::CDN->extjs( $ver, $framework, $theme, $devel );
+    }
+
+    push $resources->@*, $extjs_resources->@*;
+
+    # ExtJS locale
+    my $locale = $self->extjs_default_locale;
+
+    push $resources->@*, WWW::CDN->extjs_locale( $ver, $framework, $locale, $devel );
+
+    # ExtJS overrides
+    # <script type="text/javascript" src="/static/sencha/ext/pcore/overrides.js"></script>
 
     my $loader_path = {
         q[*]                       => q[.],
@@ -173,20 +160,10 @@ around run => sub ( $orig, $self, $req ) {
         $self->{ext_app_namespace} => $self->{path},
     };
 
-    # CloudFlare CDN
-    # https://cdnjs.com/libraries/extjs
-
-    my $extjs_ver       = '6.2.0';
-    my $extjs_framework = 'classic';
-
-    my $resources = {
-        INDEX => { title => 'Admin', },
-        ext   => {
-            extjs          => "https://cdnjs.cloudflare.com/ajax/libs/extjs/$extjs_ver/ext-all${debug}.js",
-            extjs_ux       => "https://cdnjs.cloudflare.com/ajax/libs/extjs/$extjs_ver/packages/ux/$extjs_framework/ux.js",
-            theme_css      => "https://cdnjs.cloudflare.com/ajax/libs/extjs/$extjs_ver/$extjs_framework/theme-${theme}/resources/theme-${theme}-all${debug}.css",
-            theme_js       => "https://cdnjs.cloudflare.com/ajax/libs/extjs/$extjs_ver/$extjs_framework/theme-${theme}/theme-${theme}${debug}.js",
-            locale_js      => "https://cdnjs.cloudflare.com/ajax/libs/extjs/$extjs_ver/$extjs_framework/locale/locale-@{[$self->locale]}${debug}.js",
+    my $data = {
+        INDEX     => { title => 'Admin', },
+        resources => $resources,
+        ext       => {
             api_map        => undef,
             loader_path    => mark_raw( to_json( $loader_path, readable => $devel )->$* ),
             app_namespace  => $self->{ext_app_namespace},
@@ -194,22 +171,7 @@ around run => sub ( $orig, $self, $req ) {
         },
     };
 
-    # my $resources = {
-    #     INDEX => { title => 'Admin', },
-    #     ext   => {
-    #         extjs          => "/static/sencha/ext/build/ext-${ext_framework_name}all${debug}.js",
-    #         extjs_ux       => "/static/sencha/ext/build/packages/ux/$framework/ux${debug}.js",
-    #         theme_css      => "/static/sencha/ext/build/$framework/theme-$theme/resources/theme-$theme-all${debug}.css",
-    #         theme_js       => "/static/sencha/ext/build/$framework/theme-$theme/theme-$theme${debug}.js",
-    #         locale_js      => "/static/sencha/ext/build/$framework/locale/locale-@{[$self->locale]}${debug}.js",
-    #         api_map        => undef,
-    #         loader_path    => mark_raw( to_json( $loader_path, readable => $devel )->$* ),
-    #         app_namespace  => $self->{ext_app_namespace},
-    #         viewport_class => "$self->{ext_app_namespace}.Viewport",
-    #     },
-    # };
-
-    my $index = P->tmpl->render( 'sencha/index-extjs.html', $resources );
+    my $index = P->tmpl->render( 'sencha/index-extjs.html', $data );
 
     $req->( 200, [ 'Content-Type' => 'text/html; charset=UTF-8' ], $index )->finish;
 
@@ -217,6 +179,16 @@ around run => sub ( $orig, $self, $req ) {
 };
 
 1;
+## -----SOURCE FILTER LOG BEGIN-----
+##
+## PerlCritic profile "pcore-script" policy violations:
+## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
+## | Sev. | Lines                | Policy                                                                                                         |
+## |======+======================+================================================================================================================|
+## |    1 | 132                  | RegularExpressions::ProhibitEnumeratedClasses - Use named character classes ([a-z] vs. [[:lower:]])            |
+## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
+##
+## -----SOURCE FILTER LOG END-----
 __END__
 =pod
 
